!#define FFTW

MODULE fouriermap

use mapinfo, only: map_info_type

implicit none

!integer, parameter             :: FOUR_FO=1, FOUR_2FO_FC=2, FOUR_FO_FC=3, FOUR_FC=4
!character(len=8), dimension(4) :: four_name = [ 'Fo      ', '2Fo - Fc', 'Fo - Fc ', 'Fc      ' ] 

!corr type map_info_type 
!corr    logical :: active = .false.  ! map visualization active?
!corr    integer :: ftype  = 0        ! map type? 
!corr    real    :: radius = -1       ! radius of map
!corr    real    :: sog = 0.5         ! display map above sog
!corr end type map_info_type

!corr type(map_info_type) :: mapinf

CONTAINS

!corr    subroutine init_fourier_map()
!corr!
!corr!   Initialize information about map
!corr    mapinfo%active = .false.
!corr    mapinfo%ftype = 0
!corr    end subroutine init_fourier_map
!corr
!----------------------------------------------------------------------

    subroutine jav_show_map(itypef,res,ref,map_name,sogmap,fctype,force,scalem,f000,cell)
    USE mappa, only:xmap
    USE fourier_mod
    USE reflection_type_util, only: reflection_type
    USE unit_cell
    integer, intent(in)                                       :: itypef     ! fourier type
    real, intent(in)                                          :: res
    type(reflection_type), dimension(:), intent(in), optional :: ref        ! reflections
    character(len=*), intent(in), optional                    :: map_name   ! name of map
    real, intent(in), optional                                :: sogmap     ! show only intensity > sogmap
    integer, intent(in), optional                             :: fctype     ! ref%fc id Fc (fctype=1) or Fc^2 (fctype=2)
    logical, intent(in), optional                             :: force
    logical, intent(in), optional                             :: scalem
    real, intent(in), optional                                :: f000
    type(cell_type), intent(in), optional                     :: cell
    integer                                                   :: ftype
    logical                                                   :: lforce
!
    if (itypef == 0 .and. .not.allocated(xmap)) then
!corr        mapinf = map_info_type(.true.,itypef)   ! segnala che l'utente ha richiesto la fourier
        return
    endif
!
    if (present(fctype)) then
        ftype = fctype
    else
        ftype = 1
    endif
!
    if (present(force)) then
        lforce = force
    else
        lforce = .false.
    endif
!
    if (itypef /= 0) then
        if (.not.allocated(xmap)) then
            call CheckGrid()
            call AllocaMappa(1)   
        endif
        call create_map(itypef,ref,size(ref),ftype)
        if (present(scalem)) then
            if (scalem) then
                if (present(f000) .and. present(cell)) then
                    call scalemap(f000,cell)
                endif
            endif
        endif
    endif
!
    if (present(sogmap)) then
        call VediMappa(map_name,size(ref),res,sogmap=sogmap,force=lforce)
    else
        call VediMappa(map_name,size(ref),res)
    endif
!corr    mapinf = map_info_type(.true.,itypef)
!   
    end subroutine jav_show_map
    
!----------------------------------------------------------------------

    logical function is_map_enabled(m_mapinfo)
!
!   Check if user has required a map visualization
!
    use mapinfo
    type(map_info_type), intent(in) :: m_mapinfo
    integer :: is_map_visible
!
    is_map_enabled = m_mapinfo%active .or. is_map_visible(1) == 1
!
    end function is_map_enabled

!----------------------------------------------------------------------
!corr
!corr    integer function map_type_selected()
!corr    use mapinfo
!corr!
!corr!   Which map has been required. 
!corr!
!corr    if (mapinf%ftype == 0) then
!corr        map_type_selected = FOUR_FO_FC  ! if unknown force the difference map
!corr    else
!corr        map_type_selected = mapinf%ftype
!corr    endif
!corr    end function map_type_selected
!corr
!----------------------------------------------------------------------
!corr
!corr    subroutine sogmap_init(itypef,refl,atom,sog,umapel)
!corr    USE mappa, only:xmap
!corr    USE atom_type_util
!corr    USE fourier_mod
!corr    USE reflection_type_util, only: reflection_type
!corr    integer, intent(in)                             :: itypef
!corr    type(reflection_type), dimension(:), intent(in) :: refl
!corr    type(atom_type), dimension(:), allocatable, intent(in)       :: atom
!corr    real, intent(out)                               :: sog
!corr    real, intent(out), optional                     :: umapel ! number of map unit per electrons
!corr!
!corr    if (.not.allocated(xmap)) then
!corr        call CheckGrid()
!corr        call AllocaMappa(1)   
!corr    endif
!corr    call create_map(itypef,refl,size(refl),1)
!corr    sog = maxval(xmap) * 0.8
!corr    if (present(umapel)) then
!corr        if (numatoms(atom) > 0) then
!corr            umapel = maxval(xmap) / maxval(atom%z())
!corr        else
!corr            umapel = 1.0
!corr        endif
!corr    endif
!corr    !if (code == 1) then
!corr    !elseif (code == 2) then
!corr    !    call create_map(FOUR_FC,refl,size(refl),1)
!corr    !    zval(:) = atomic_number(atom)
!corr    !    sog = maxval(xmap) / maxval(zval)
!corr    !    sog = maxval(xmap) * 0.08
!corr    !endif
!corr       !write(0,*)'sog=',maxval(xmap),sog
!corr!
!corr    end subroutine sogmap_init
!corr
!----------------------------------------------------------------------

    subroutine create_map(itypef,refl,nrefsize,fctype)
    USE mappa
    USE fourier_mod
    USE reflection_type_util, only: reflection_type
    integer, intent(in)                             :: itypef
    type(reflection_type), dimension(:), intent(in) :: refl
    integer, intent(in)                             :: nrefsize
    integer, intent(in)                             :: fctype
    integer, dimension(3,nrefsize)                  :: hkl
    real, dimension(nrefsize)                       :: wei
    real, allocatable, dimension(:)                 :: fcoef
    integer                                         :: i
!
    do i=1,nrefsize
       hkl(:,i) = refl(i)%hkl(:)
    enddo
    wei(:) = 1.0
    call fourier_coef(fcoef,refl,nrefsize,itypef,fctype)
#ifdef FFTW
    call MakeMap(hkl,fcoef,refl(:)%ph,nrefsize,0,xmap,FORTH,BACK)
      !       call invmap(hkl, iphi2, fop2, nrefsize, np(1), np(2), np(3), BACK)
      !   do i=1,nrefsize
      !      write(70,*)fop2(i), fcoef(i),iphi2(i),refl(i)%ph
      !   enddo
#else
    call MakeMap(hkl,fcoef,refl(:)%ph,wei,nrefsize,0,xmap) 
#endif
!
    end subroutine create_map

!----------------------------------------------------------------------

    subroutine VediMappa(namem,nref,resm,sogmap,force)
    USE Mappa
    USE stat
    USE fourier_mod
                 USE stat
!   
    implicit none
    real, intent(in), optional    :: sogmap
    character(len=*), intent(in)  :: namem
    integer, intent(in)           :: nref
!!!FIXME - spiegare force
    logical, intent(in), optional :: force
    real                          :: denmin, denmax, avden, stden
    integer                       :: npunt
    real                          :: sog
    real, intent(in) :: resm
!   
    !resm = 2.0
    call maxmin3d(denmin,denmax,avden,stden,npunt,0)
!!!!    sogmap = 100  ! mostra tutto cio' che è al di sopra di sogmap (stden*1.5)
    if (present(sogmap)) then
        sog = sogmap
        if (force) then
            if (sogmap > denmax) sog = denmax*0.5
        endif
!corr        write(0,'(a,5(a,f10.3))')'mappa:','min=',denmin,' max=',denmax,' ave=',avden,' std=',stden,' sog=',sog
        call PassaMappaParz(trim(namem)//char(0), 1, 1, np, nref, resm, denmin, denmax, avden, sog/1.5, 1)
    else
        call PassaMappaParz(trim(namem)//char(0), 1, 1, np, nref, resm, denmin, denmax, avden, stden, 1)
    endif
!   
    end subroutine VediMappa


!----------------------------------------------------------------------

    subroutine MakeFour(icalfo,npikfo,iweifo,iselect_ref,preff,showfourier,ier)
    USE General
    USE Molcom      
    USE nr
    USE PhaseModule
    USE Mappa
    USE variables, only:atm,ref,dataset,cryst
    USE atom_type_util
    USE fourier_mod
    USE reflection_type_util
    USE datasetmod
    USE counts
    USE mapinfo
    implicit none
    integer, intent(in) :: showfourier
    common /searchfour/ isearchfo
    integer :: isearchfo
    integer, intent(in)                              :: icalfo
    integer, intent(in)                              :: npikfo
    integer, intent(in)                              :: iweifo
    integer, intent(in)                              :: iselect_ref
    real, intent(in)                                 :: preff
    integer, intent(out)                             :: ier
    type(reflection_type), allocatable, dimension(:) :: refl
    real, dimension(nrefl)                           :: wwt
    integer, dimension(nrefl)                        :: iorder
    real, allocatable, dimension(:)                  :: frvet
    integer, allocatable, dimension(:,:)             :: hkl
    integer                                          :: i 
    integer                                          :: mode,jpatt
    integer                                          :: numbMax
    integer                                          :: nExitCode
    type(atom_type), allocatable, dimension(:)       :: atmold
    real :: sogfmap   !, umapel    !!!!, maxmap1
    real :: maxmap,minmap,avem,stdm
    real :: f000,res
!
    ier = 0
    if (is_extraction) then
        if (numatoms(atm) == 0 .and. (icalfo == FOUR_FO_FC .or. icalfo == FOUR_2FO_FC)) then
            call MsgWinErr('ERROR','F(calc) are not available!',ERR_WINDOW,nExitCode)
            return
        endif
        if (numatoms(atm) > 0) call fcalcang(ref,atm,spacegr,celem)
        call copy_ref(refl,ref)
            !ref%fo = ref%fv
    else
        call MsgWinErr('ERROR','F(obs) are not available!',ERR_WINDOW,nExitCode)
        return
    endif
!
!   Salvo modello corrente per confronto
    call copy_atoms(atmold,atm)
!
!   Calcolo peso wwt
!   iweifo = 0 --> wwt = 1.
!   iweifo = 1 --> wwt = peso di tipo Sim
    if (iweifo == 1.or.iselect_ref == 2) then
        call simweight(refl,wwt)
    else
        wwt(:) = 1.0
    endif
!
!   Ordina riflessi e pesi
    select case (iselect_ref) 
       case (1)          ! ordina in base all'Fo
         call indexx(refl(:)%fo,iorder)
         refl(:) = refl(iorder)
         if (iweifo == 1) then
             wwt(:) = wwt(iorder)
         else
             wwt(:) = 1.0
         endif

       case (2)          ! ordina in base al peso
         call indexx(wwt,iorder)
         refl(:) = refl(iorder)
         if (iweifo == 1) wwt(:) = wwt(iorder)

    end select 
!
!   Seleziona una percentuale di riflessi pari a preff
    numbMax = nint(nrefl*preff/100.)
    if (numbMax.lt.1) numbMax = 1
    if (numbMax.gt.nrefl) numbMax = nrefl
!
    call AllocaMappa(1)
!
    write(lo,'(/10x,a)')'***   A Fourier map with ('//trim(four_name(icalfo))//') coefficients ***'
    select case (icalfo) 
        case (FOUR_FO)        ! Fo
          aperc(1) = numatoms(atm) + npikfo
        case (FOUR_2FO_FC)        ! 2Fo - Fc 
          aperc(1) = numatoms(atm) + npikfo
        case (FOUR_FO_FC)        ! Fo - Fc
          aperc(1) = npikfo
    end select
    write(lo,'(20x,a)') " ***   has been required   ***"
    write(lo,'(/7x,a,i0,a,i0,a)')'Number of used reflections: ',numbMax,' reflections (on ',nrefl,')'
!corr               umapel = 1.0    !!!!FIXME-non funziona
!corr      if (showfourier == 1) then
!corr          call  sogmap_init(FOUR_FC,refl,atm,sogfmap,umapel) 
!corr   !!!!test       maxmap1 = maxval(xmap)
!corr      endif
!
!     Esegue Fourier
      jpatt = 0
      krecy = 0
      allocate(hkl(3,numbMax))
      do i=1,numbMax
         hkl(:,i) = refl(i)%hkl
      enddo
      call fourier_coef(frvet,refl,numbMax,icalfo,1)
#ifdef FFTW
      call MakeMap(hkl,frvet,refl(:)%ph,numbMax,jpatt,xmap,FORTH,BACK)
#else
      call MakeMap(hkl,frvet,refl(:)%ph,wwt,numbMax,jpatt,xmap)
#endif
!
      if (icalfo == FOUR_FO) then
          f000 = get_f000(atm,celem,cryst(1)%spg,dataset(1)%radtype)
          write(lo,'(7x,a,f10.3,a)')'F000:        ',F000,' e'
      else
          f000 = 0.0
      endif
!
      call scalemap(f000,cryst(1)%cell)   ! scale map in eV/A^3
      call maxmin3d(minmap,maxmap,avem,stdm,id=0)
      write(lo,'(7x,a,f10.3,a)')'Density max: ',maxmap,' e/A**3'
      write(lo,'(7x,a,f10.3,a)')'Density min: ',minmap,' e/A**3'
      write(lo,'(7x,a,f10.3,a)')'Density rms: ',stdm,  ' e/A**3'
!
      if (showfourier == 1) then
          call map_set_visible(1,1)   ! force map visualization
          sogfmap = 1  ! display maps for electrons > 1
          if (ndataset(dataset) > 0) then
              res = dvalue(dataset(1)%tmax,dataset(1)%wave(1))
          else
              res = 2.0
          endif
          !mapinf = map_info_type(.true.,icalfo) 
          call jav_show_map(0, res, refl, map_name = trim(four_name(icalfo))//' map',sogmap=sogfmap,force=.false.)
      endif
!
!     Esegue peak search
      isearchfo = 1
      mode = 0
      call Search99(mode,ier,numbMax)
      !call four_point(xyzs(:,:nats),ref,frvet,cryst(1)%cell,cryst(1)%spg,dataset(1)%radtype)
      isearchfo = 0
!
!     Confronta vecchio e nuovo modello
      call labelx(atmold,cryst(1)%cell,cryst(1)%spg,npikfo,ier)
      call to_coord(1)
!
      end subroutine makefour

!----------------------------------------------------------------------

      subroutine fourier_coef(fcoef,refl,nrefuse,ftype,fctype)
!
!     Calculate coefficients for Fourier
!
      USE arrayutil
      USE reflection_type_util, only: reflection_type
      USE mapinfo
      real, dimension(:), allocatable, intent(out)    :: fcoef   ! coefficients
      type(reflection_type), dimension(:), intent(in) :: refl    ! reflections
      integer, intent(in)                             :: ftype   ! Fourier type
      integer, intent(in)                             :: fctype  ! 1 -> Fc, 2 -> Fc^2
      integer, intent(in)                             :: nrefuse ! number of used reflections for fourier
      real                                            :: scal
      real, dimension(size(refl))                     :: fq 
!
!     compute scale
      if (fctype == 1) then
          scal =  sum(refl(:)%fc) / sum(refl(:)%fo)  
      else
          fq(:) = sqrt(refl(:)%fc)                    
          scal =  sum(fq(:)) / sum(refl(:)%fo)
      endif
!
      call resize_array(fcoef,nrefuse)
!
      select case(ftype)
         case (FOUR_FO)
           fcoef(:) = scal*refl(:nrefuse)%fo
         case (FOUR_2FO_FC)
           if (fctype == 1) then
               fcoef(:) = 2*scal*refl(:nrefuse)%fo - refl(:nrefuse)%fc
           else
               fcoef(:) = 2*scal*refl(:nrefuse)%fo - fq(:nrefuse)
           endif
         case (FOUR_FO_FC)
           if (fctype == 1) then
               fcoef(:) = scal*refl(:nrefuse)%fo - refl(:nrefuse)%fc
           else
               fcoef(:) = scal*refl(:nrefuse)%fo - fq(:nrefuse)
           endif
         case (FOUR_FC)
           if (fctype == 1) then
               fcoef(:) = refl(:nrefuse)%fc
           else
               fcoef(:) = fq(:nrefuse)
           endif
      end select
!
      end subroutine fourier_coef

!----------------------------------------------------------------------

   subroutine labelx(atmold,cell,spg,nPeaksToFind,ier)
   USE General, only: lo, StructureName, celem
   USE Symmetry, only: ddmin
   USE Molcom, only: It
   USE sinterface
   USE ilmviewmod
   USE variables
   USE atom_type_util
   USE fragmentmod
   USE connect_mod
   USE unit_cell
   USE PhaseModule, only:biso
   USE strutil
   USE cgeom
   USE spginfom
!
   implicit none
   type(atom_type), dimension(:), allocatable, intent(in) :: atmold
   type(cell_type), intent(in)                            :: cell
   type(spaceg_type), intent(in)                          :: spg
   integer, intent(in)                                    :: nPeaksToFind
   type(fragment_type), dimension(:), allocatable         :: fragm
   integer                                                :: nfrag
   real, dimension(3)                                     :: xn,xo,xeq,s
   real, dimension(2)                                     :: vet
   real, dimension(:), allocatable                        :: tempdist
   real, allocatable, dimension(:)                        :: distminv
   integer, allocatable, dimension(:)                     :: kminv,selected,npunt
   integer                                                :: i, j, k, ifi, kt, jpunt, ifin
   integer                                                :: Naccept
   integer                                                :: ier
   integer                                                :: iExitC,MultiSel,NumCan
   real                                                   :: d, distm
   integer                                                :: iat, nnzo
   integer                                                :: nat_old, nat_new
   type(atom_type), dimension(:), allocatable             :: atmnew
   logical                                                :: kpr = .true.
   integer                                                :: new_peaks
   real                                                   :: coef
   character(len=:), allocatable                          :: msg
!corr   type(cell_type) :: cellt
!
   nat_old = numatoms(atmold)
   call xyzs_to_atom(atmnew)
   nat_new = numatoms(atmnew)
   call atom_string(atmnew,celem)  ! peaks will be as Q1, Q2, ...
   atmnew(:)%biso=biso
   if (kpr) then
       call print_atoms(atmold,title='Old atoms list')
       call print_atoms(atmnew,title='Fourier peaks',height=.true.)
   endif
   allocate(tempdist(nat_new))
   allocate(npunt(nat_new),source=0)
   vet(:) = (/1.0,-1.0/)
!corr   ifin=icent+1
!corr   qq(:) = 0.0
   if (spg%symcent == 1) then
       ifin = 2
   else
       ifin = 1
   endif
!corr   cellt = set_cell_type(cell)
   do i=1,nat_old
      !nnzo=iabs(atmold(i)%nz)/nzConst
      nnzo=atmold(i)%kscatt()
      xo(:) = atmold(i)%xc(:)
      distm=9999
      jpunt=0
      do j=1,nat_new
!corr         if (qq(j).lt.0.5) then
         if (npunt(j) == 0) then
             xn(:) = atmnew(j)%xc(:)
             do kt=1,spg%ncoper
                do ifi=1,ifin
                   coef=vet(ifi)
                   do k=1,spg%nsym
                      call xequi(xo,k,coef,xeq,kt)
!                      call xdist(xn,xeq,d,s)
                      call xdisteqs(xn,xeq,cell%get_g(),d,s)
                      if (d.lt.ddmin) then
                          distm=d
                          !jpunt=j
                          !qq(jpunt) = i
                          !tempdist(jpunt) = d
                          npunt(j) = i
                          tempdist(j) = d
                      endif
                   enddo
                enddo
             enddo
         endif
      enddo
   enddo
!
   if (kpr) then
       do j=1,nat_new
!corr          i=nint(qq(j))
!test for real              write(0,*)'ATM=',j,atmnew(j)%inte
          i=npunt(j)
          if (i > 0) then
              write(lo,'(2x,a,f8.2,4f10.3,10x,a8,i4,4f10.3,1x,f10.3)')             &
              atmnew(j)%lab,atmnew(j)%inte,atmnew(j)%xc,atmnew(j)%biso,    &
              atmold(i)%lab,i,atmold(i)%xc(:),atmold(i)%biso,tempdist(j)
          else
              !corr write(lo,'(2x,a,i5,4f10.3)') atmnew(j)%lab,nint(atmnew(j)%inte),atmnew(j)%xc,atmnew(j)%biso
              write(lo,'(2x,a,f8.2,4f10.3)') atmnew(j)%lab,atmnew(j)%inte,atmnew(j)%xc,atmnew(j)%biso
          endif
       enddo
   endif
   new_peaks = 0
!corr   nequal = 0
!corr   sumold = 0.0
!corr   sumnew = 0.0
!corr!
!corr!  Scale intensities
!corr   scalatom = 1.0
!corr   if (icalfo /= FOUR_FO_FC) then
!corr       do j=1,nat_new
!corr          i=npunt(j)
!corr          if (i > 0) then
!corr              nequal = nequal + 1
!corr              sumold = sumold + atmold(i)%inte
!corr              sumnew = sumnew + atmnew(j)%inte
!corr          endif
!corr!          new_peaks = new_peaks + 1
!corr       enddo
!corr       if (nequal > 0 .and. sumnew > 0) scalatom = sumold/sumnew
!corr   endif
!
   do j=1,nat_new
!
!     accept the new peak only if has no counterpart
!     in the atom list
!
      i=npunt(j)
      if (i == 0) then
          new_peaks = new_peaks + 1
          !atmnew(new_peaks)%xc = atmnew(j)%xc
          atmnew(new_peaks) = atmnew(j)
          atmnew(new_peaks)%biso = biso
      endif
   enddo
!corr!
!corr! Riscala intensita
!corr!
!corr   scalatom = 1.0
!corr  !!!!FIXME-non funziona                     atmnew(:)%inte = atmnew(:)%inte / umapel
!corr!corr   if (nequal.gt.0.and.icalfo.ne.3) then
!corr   if (nequal > 0 .and. icalfo /= FOUR_FO_FC) then
!corr       if (sumnew.gt.0.0) scalatom = sumold/sumnew
!corr   endif
!
   if (new_peaks > nPeaksToFind) new_peaks = nPeaksToFind
!
   nat_new = new_peaks + nat_old
   call resize_atoms(atmnew,nat_new)
   if (new_peaks > 0) then
       write(lo,"(10x,i5, '  new peak(s) found')") new_peaks
!corr  265     format(  10x,i5, '  new peak(s) found')
       call calcola_occ(atmnew(:new_peaks),spg,cell)
!
!         move atoms at the end of arrays
!         to make room for old atoms
       atmnew(nat_old+1:) = atmnew(:new_peaks)
   endif
!
!  copy old atoms at the beginning of arrays
   do i=1,nat_old
      atmnew(i) = atmold(i)
   enddo
!corr   atmnew(:nat_old) = atmold(:)
!
   if (new_peaks > 0) then
       call copy_atoms(atm,atmnew)
!
!      Frammenta il modello
       call get_fragments(atm,cell,lconn,nfrag,fragm)
!
!      Raggruppa i frammenti 
       call regroup_fragment(fragm,atm,cell,spg)
!corr!
!corr!      Disegna il modello e i nuovi atomi come ghost
!corr       if (.not.jav_opened())call ilmview_open(StructureName)
!corr       call jav_set_modify(0)
!corr       call ilmview_draw(atm,lconn,celem,set_cell_type(cell),spacegr)
!
!      Calcola distanza minime
       !call copy_atoms(atmc,atm)
       !call frac_to_cart(atmc,orthomatrix(cell))
       allocate(distminv(new_peaks),kminv(new_peaks))
       if (nat_old > 0) then
           do i=1,new_peaks
              iat = nat_old+i
!!!!test
          call get_minimum_distance_sym(atm(iat),atm(:nat_old),cell,spg,kminv(i),distminv(i),xeq)
          atm(iat)%xc = xeq
          !write(0,*)'At=',trim(atm(iat)%lab),distmin,trim(atm(kmin)%lab)
!!!!test
          !    call get_minimum_distance(atmc(iat),atmc(:nat_old),kmin,distmin)
          !    distminv(i) = distmin
          !    kminv(i) = kmin          !!mr
              !write(It(i),'(a,i0,4x,a,f0.3)')trim(atm(iat)%lab)//'  I=', nint(atm(iat)%inte),  &
              write(It(i),'(a,f8.2,4x,a,f0.3)')trim(atm(iat)%lab)//'  I=', atm(iat)%inte,  &
                'dmin('//trim(atm(iat)%lab)//'-'//trim(atm(kminv(i))%lab)//')=' ,distminv(i)
               It(i) = trim(It(i))//char(0)
           enddo
       else
           do i=1,new_peaks
              distminv(i) = 0
              kminv(i) = i
              !write(It(i),'(a,i0,4x,a,f0.3)')trim(atm(i)%lab)//'  I=', nint(atm(i)%inte),  &
              write(It(i),'(a,f8.2,4x,a,f0.3)')trim(atm(i)%lab)//'  I=', atm(i)%inte,  &
                'dmin('//trim(atm(i)%lab)//'-'//' - '//')=' ,distminv(i)
               It(i) = trim(It(i))//char(0)
           enddo
       endif
!
!      Disegna il modello e i nuovi atomi come ghost
       if (.not.jav_opened())call ilmview_open(StructureName)
       call jav_set_modify(0)
       call ilmview_draw(atm,lconn,celem,cell,spg)
!
       call ClearMessage()
! --
       msg = trim(i_to_s(new_peaks))//' new peaks found.'//char(10)//  &
             'Check Intensities &amp; distances and select peaks to retain.'//char(0)
       NumCan = new_peaks
       MultiSel = 1
       allocate (selected(NumCan))
       selected(:) = 0
       call c_NewPeekList('Select peaks to accept'//char(0), new_peaks,     &
                          MultiSel, selected,iExitC, nat_old, msg, atm, distminv, kminv)
       if (iExitC == 3) then
           selected(:) = 0
       elseif (iExitC.eq.2) then
           selected(:) = 1
       endif
       Naccept = 0
       do i=1, NumCan
          if (selected(i).eq.1) then
              Naccept = Naccept + 1
              atm(nat_old+Naccept) = atm(nat_old+i)
          endif
       enddo
       if (Naccept == 0) ier = 1 ! segnala che non c'e' modifica
       call resize_atoms(atm,nat_old+Naccept)
       call ilmview_draw(atm,lconn,celem,cell,spg)
       call jav_set_modify(1)
   else
       ier = -1
       call OpenWarning('No new peaks found !!!')
   endif
   call atom_to_xyzs(atm)
!
   end subroutine labelx

!corr!----------------------------------------------------------------------------------------------------
!corr
!corr   real function radmap_from_asu(cell,spg) result(radmap)
!corr!
!corr!  Set radius of map from size of asymmetric unit
!corr!
!corr   use unit_cell
!corr   use spginfom
!corr   type(cell_type), intent(in)   :: cell
!corr   type(spaceg_type), intent(in) :: spg
!corr   integer                       :: kmin
!corr   kmin = maxloc(spg%asulim,dim=1)
!corr   radmap = cell%get_par(kmin)*spg%asulim(kmin) / 2
!corr   end function radmap_from_asu
!corr
!corr!----------------------------------------------------------------------------------------------------
!corr
!corr   real function radmap_from_atoms(atom, cell, frag)  result(radmap)
!corr!
!corr!  Ser radius of map from centre of mass of molecules
!corr!
!corr   use atom_type_util
!corr   use fragmentmod
!corr   use unit_cell
!corr   type(atom_type), dimension(:), allocatable, intent(in)     :: atom
!corr   type(cell_type), intent(in)                                :: cell
!corr   type(fragment_type), dimension(:), allocatable, intent(in) :: frag
!corr   type(atom_type), dimension(:), allocatable                 :: atomcart
!corr   integer                                                    :: i
!corr   real                                                       :: maxradmap
!corr!
!corr   radmap = -1
!corr   if (numatoms(atom) == 0) return
!corr   if (numfragments(frag) == 0) return
!corr   call frac_to_cart_copy(atom,atomcart,cell%get_ortom())
!corr   maxradmap = tiny(1.0)
!corr   do i=1,numfragments(frag)
!corr      call get_radius_molecule(atomcart(frag(i)%pos),radmap)
!corr      if (radmap > maxradmap) maxradmap = radmap
!corr   enddo
!corr   radmap = maxradmap
!corr!
!corr   end function radmap_from_atoms
!corr
!corr!----------------------------------------------------------------------------------------------------

   subroutine simweight(refl,sim,eoo,ecc)
   USE General, only : bt,celem,neutro,spacegr
!corr   USE General, only : ff,bt,celem,neutro,   spacegr
!corr   USE Symmetry, only : pts
   USE reflection_type_util, only: reflection_type,get_norm
!corr   USE elements, only: numelem,at_scatt
!
   type(reflection_type), dimension(:), intent(in) :: refl
   real, dimension(:),intent(out)                  :: sim
   real, dimension(:),intent(out), optional        :: eoo,ecc
   real, dimension(size(refl))                     :: eo,ec
!corr   integer                                         :: nr
   real                                            :: bes
!corr   real                                            :: sumff,den,bes
!corr   real                                            :: eps
!corr   integer                                         :: multi
   integer                                         :: i   !,ij
!
   call get_norm(refl,spacegr,celem,bt,neutro,eo,ec)
!corr   nr = size(refl)
   do i=1,size(refl)
!corr      ff(:numelem(celem)) = at_scatt(celem,refl(i)%slaq,neutro)
!corr      sumff = 0.0
!corr      do ij=1,numelem(celem)
!corr         sumff = sumff + celem(ij)%nw*(ff(ij)**2)
!corr      enddo
!corr      call cmult2002(refl(i)%hkl,eps,multi)
!corr      den = sqrt((exp(-2.0*bt*refl(i)%slaq))*sumff*eps)
!corr      ec(i)=refl(i)%fc/(sqrt(pts)*den)
!corr      eo(i)=refl(i)%fo/(sqrt(pts)*den)
      sim(i)=bes(2.0*eo(i)*ec(i))
!corr       write(71,'(a,6f10.4)')'EO=',eo(i),eotest(i),ec(i),ectest(i),abs(eo(i)-eotest(i)),abs(ec(i)-ectest(i))
   enddo
   where(eo < 0.1 .and. ec < 0.1)sim=0.0
   if (present(eoo) .and. present(ecc)) then
       eoo(:) = eo(:)
       ecc(:) = ec(:)
   endif
!
   end subroutine simweight

END MODULE fouriermap

!----------------------------------------------------------------------

    subroutine Lanciajav(itypef, iAction, ierf)
    USE variables, only: cryst, dataset
    USE ilmviewmod
    USE fouriermap
    USE fourier_mod
    USE atom_type_util
    USE reflection_type_util
    USE prog_constants
    USE datasetmod
    USE counts
    USE mapinfo
    implicit none
    integer                                          :: itypef, iAction, ierf
    integer, dimension(3)                            :: ihmx
    type(reflection_type), dimension(:), allocatable :: ref
    integer                                          :: nph, nref
    real                                             :: wave, f000, res
    integer                                          :: radtype
    real, parameter                                  :: DMAX = 1.7
!   
    ierf = 0
    nph = active_phase
    if (cryst(nph)%natoms() == 0) then
        ierf = 1
        return
    endif
    if (ndataset(dataset) > 0) then
        wave = dataset(1)%wave(1)
        radtype = dataset(1)%radtype
        res = dvalue(dataset(1)%tmax,dataset(1)%wave(1))
    else
        wave = DEF_WAVE
        radtype = RX_SOURCE
        res = DMAX
    endif
    if (cryst(nph)%numref() == 0) then
        if (itypef == FOUR_FC) then
            call create_reflections(0.,thvalue(res,wave),cryst(nph)%cell%get_par(),cryst(nph)%spg,[wave],ref,nref,ihmx)
            if (nref == 0) then
                ierf = 2
                return
            endif
        else
            ierf = 2
            return
        endif
    else
        call cryst(nph)%get_reflections(ref)
    endif
!  
!   Disable 'Show Fourier Map' required?
    if (itypef == 0 .and. iAction == 0) then
        call map_set_visible(1,0)  
        return
    endif
!
    if (ierf == 0) then
        call ilmview_open()
        !corr cryst(nph)%mapinfo = map_info_type(.true.,itypef)
        cryst(nph)%mapinfo%ftype = itypef
        call fcalcang(ref,cryst(nph)%at,cryst(nph)%spg,cryst(nph)%elem)
        if (cryst(nph)%mapinfo%radius < 0) then
            cryst(nph)%mapinfo%radius = radmap_from_asu(cryst(nph)%cell,cryst(nph)%spg)
            call map_set_rad(cryst(nph)%mapinfo%radius)
        endif
          !write(0,*)'RAD=',cryst(nph)%mapinfo%radius
        call map_set_centre(baricentro(cryst(nph)%at))
        call map_set_visible(1,1)   ! force map visualization
        if (itypef /= 0) then
            f000 = get_f000(cryst(nph)%at,cryst(nph)%elem,cryst(nph)%spg,radtype)
            call jav_show_map(itypef,res,ref, 'Fourier map',cryst(nph)%mapinfo%sog,itypef,  &
                 scalem=.true.,f000=f000,cell=cryst(1)%cell)
        else
            call jav_show_map(itypef,res,ref, 'Fourier map')
        endif
    endif
!   
    end subroutine Lanciajav

!----------------------------------------------------------------------
!
!    real function GetXMap(i, j, k)
!    USE Mappa
!!   
!    implicit none
!    integer       :: i, j, k
!!   
!    GetXMap = xmap(i,j,k)
!!   
!    return
!    end
!
!----------------------------------------------------------------------
