#include <iostream>
#include <libcomune.h>
#include <libmath.h>
#include <libvector.h>
#include <libmatrix.h>

#include <openbabel/babelconfig.h>
#ifndef OB_VERSION_CHECK
#define OB_VERSION_CHECK(major, minor, patch) ((major<<16)|(minor<<8)|(patch))
#endif
#define MYSIZE 32768

#if (OB_VERSION >= OB_VERSION_CHECK(3, 0, 0))
#include <openbabel/elements.h>
#include <openbabel/generic.h>
#include <openbabel/bond.h>
#endif

#include <openbabel/obconversion.h>
#include <openbabel/mol.h>
#include <openbabel/forcefield.h>
#include <openbabel/builder.h>

using namespace std;
using namespace OpenBabel;

extern "C" {
void passa_atomo_al_fortran_(int *i, int *nat, atom_type *atom);
void passa_legame_al_fortran_(int *i, int *nb, f_bond_type *leg);
void passa_cella_al_fortran_(float cella[]);
void passa_spg_al_fortran_(int *spgn);
void passa_spgnam_al_fortran_(char *spgn, size_t l);
int  getatomicnumber_(atom_type *atomo);
void jav_optimize_geometry_mopac_(int *code, int *whichModel, atom_type *kptra, int *nptra);
}

int GetNz(int atomic_num);

typedef struct {
                 int id;            // forcefield method
                 int algo;          // optimization algorithm
                 int steps;         // maximum number of iterations
                 int constr_atoms; // if 1 constraints on positions of atoms is enabled
                 int log;          // set log file
                 double conv;      // energy convergence criteria
               } force_field_type;

enum {FF_MMFF94=1, FF_GHEMICAL, FF_UFF};
enum {CG_ALGO=1, SD_ALGO};

//const string labcode[] = {"MMFF94", "Ghemical", "UFF"};
static const char *labcode[] = {"MMFF94", "Ghemical", "UFF"};

namespace obutils
{

  vector<atom_type> atoms;
  vector<f_bond_type> bonds;
  int natoms, nbonds, spgnum;
  float Cell[6] = {0, 0, 0, 0, 0, 0};
  char spg[16] = " ";
  OBConversion conv1;
  void PrintAtoms(OBMol mol)
  {
     OBAtom *at;
     vector<OBAtom*>::iterator atom_it;
     int i = 1;
     cout << "************ Atoms *******************\n";
     for (at = mol.BeginAtom(atom_it);at;at = mol.NextAtom(atom_it))
     {
         if(at != NULL)
         {
            double *v = (double *) malloc(3*sizeof(double));
            v = at->GetCoordinate();
            cout << i << ": " << v[0] << ", " << v[1] << ", " << v[2] << "\n";
            i++;
         }
     }
     cout << "**************************************\n";
  }

  void PrintLegs(OBMol mol)
  {
     OBBond *bond;
     vector<OBBond*>::iterator bond_it;
     int i = 1;
     cout << "************ Bonds *******************\n";
     for (bond = mol.BeginBond(bond_it);bond;bond = mol.NextBond(bond_it))
     {
         if(bond != NULL)
         {
            cout << i << ": " << bond->GetBeginAtomIdx() << ", " << bond->GetEndAtomIdx() << ", " << bond->GetLength() << "\n";
            i++;
         }
     }
  }

  void FromMolToAtm(OBMol mol, bool fromFile=true)
  {
     natoms = mol.NumAtoms();
     nbonds = mol.NumBonds();
     atoms.clear();
     bonds.clear();

     OBAtom *at;
     vector<OBAtom*>::iterator atom_it;
     for (at = mol.BeginAtom(atom_it);at;at = mol.NextAtom(atom_it))
     {
         atom_type atnew = {  {0.f, 0.f, 0.f} , // x,y,z
                              0.f,            // fattore termico
                              {0.f, 0.f, 0.f,
                              0.f, 0.f, 0.f},  //  fattori termici anisotropi: b11,b22,b33,b12,b13,b23
                              1.f,            // occupanza chimica
                              1.f,            // occupanza cristall.
                              1.f,            // intensita di picco
                              0,              // nz
                              " ",            // label
                              {0,0,0,0,0 },     // posizione nella matrice jac. del parametro se affinato; 0 se non affinato
                              0,              // pointer to table of elements
                              {0.f, 0.f, 0.f}, // standard deviation on x,y,z
                              0.f,            // standard deviation on biso
                              0.f,            // standard deviation on och
                              0,              // corrisponding atom in the a.u.
                              {1, 0, 0, 0},   // symmetry operator
                              0,              // dynamical occupancy correction
                              {' '}           // protein info
                            };

         if(at != NULL)
         {
            double *v = (double *) malloc(3*sizeof(double));
            v = at->GetCoordinate();
            if (at->HasData("_atom_site_label"))
            {
                OBPairData *label = (OBPairData *)at->GetData("_atom_site_label");
                if(label)
                {
                   size_t max_l = MRmin(16, label->GetValue().length());
                   for(size_t i=0; i< max_l; i++)
                       atnew.lab[i] = label->GetValue().at(i);
                }
            }
            atnew.nz   = at->GetAtomicNum();  // questo è sbagliato
            atnew.ptab = at->GetAtomicNum();
            atnew.inte = (float) at->GetAtomicNum();
            for(int i=0; i<3; i++)
                atnew.xc[i] = (float) v[i];
            atoms.push_back(atnew);
            int k = at->GetIdx();
            if(fromFile)
               passa_atomo_al_fortran_(&k, &natoms, &atnew);
         }
     }

     OBBond *bond;
     f_bond_type bond_new = {
                              0, 0,     //atomi legati
                              0.f,      //distanza corrente
                              0.3f,     //sigma
                              0         //bond order
                             };

     vector<OBBond*>::iterator bond_it;
     for (bond = mol.BeginBond(bond_it);bond;bond = mol.NextBond(bond_it))
     {
         if(bond != NULL)
         {
            bond_new.n1   = bond->GetBeginAtomIdx();
            bond_new.n2   = bond->GetEndAtomIdx();
            bond_new.dist = (float)bond->GetLength();
            bond_new.ord  = bond->GetBondOrder();
         }
         bonds.push_back(bond_new);
         int k = bond->GetIdx() + 1;
         if(fromFile)
            passa_legame_al_fortran_(&k, &nbonds, &bond_new);
     }
  }

  OBForceField *find_force_field(const char *which)
  {
     return (OBForceField::FindForceField(which));
  }


#define BOHR_TO_ANGSTROM 0.5291772108
#define ANGSTROM_TO_BOHR 1.0 / BOHR_TO_ANGSTROM

  bool ReadABINIT(ifstream *ifs, bool *err1, OBMol *mol)
  {
     *err1 = false;
     char buffer[MYSIZE];
     //OBMol mol;
     double x,y,z;
     double acell[3];
     OBAtom *atom;
     int natom = 0;
     vector<string> vs;
     vector<int> atomicNumbers, atomTypes;
     vector<vet3> atomPositions;
     vector<double> energies;
     vector3 translationVectors[3];
     int numTranslationVectors = 0;
     int symmetryCode = 0;


     mol->Clear();
     mol->BeginModify();
     bool is_ABINIT_out = false;
     for(int k=0; k<10; k++)
     {
         if(ifs->getline(buffer,MYSIZE))
         {
            if (strstr(buffer,"ABINIT") != NULL)
            {
                is_ABINIT_out = true;
               break;
            }
         }
     }
     ifs->clear();
     ifs->seekg(0, ifs->beg);
     if(!is_ABINIT_out)
        return false;

     while (ifs->getline(buffer,MYSIZE) && !(*err1))
     {
        if (strstr(buffer, "acell"))
        {
          tokenize(vs, buffer);
          if (vs.size() < 4)
            continue; // invalid line

          for (int i = 0; i < 3; ++i) 
            acell[i] =  atof(vs[i+1].c_str());
        }
        //else if (strstr(buffer, "artesian coordinates"))
        else if (strstr(buffer, " xcart "))
        {
          double unit = BOHR_TO_ANGSTROM;
          if (strstr(buffer, "ngstrom"))
            unit = 1.0; // no conversion needed

          // ifs->getline(buffer,MYSIZE);
          tokenize(vs, buffer);
          for(int kat=0; kat<natom; kat++)
          {
            int ini = 0;
            if(vs.size() == 4)
               ini = 1;
            else if(vs.size() < 3)
            {
               *err1 = true;
               break;
            }
            x = atof(vs[ini].c_str()) * unit;
            y = atof(vs[ini+1].c_str()) * unit;
            z = atof(vs[ini+2].c_str()) * unit;

            vet3 v = {x, y, z};
            atomPositions.push_back(v);
            // get next line
            ifs->getline(buffer,MYSIZE);
            tokenize(vs, buffer);
          }
        }
        else if (strstr(buffer, "natom"))
        {
          tokenize(vs, buffer);
          if (vs.size() != 2)
            continue;
          natom = atoi(vs[1].c_str());
        }
        else if (strstr(buffer, "rprim"))
        {
          numTranslationVectors = 0;
          int column;
          for (int i = 0; i < 3; ++i) {
            tokenize(vs, buffer);
            if (vs.size() < 3)
              break;

            // first line, rprim takes up a token
            if (i == 0)
              column = 1;
            else
              column = 0;

            x = atof((char*)vs[column].c_str()) * BOHR_TO_ANGSTROM;
            y = atof((char*)vs[column+1].c_str()) * BOHR_TO_ANGSTROM;
            z = atof((char*)vs[column+2].c_str()) * BOHR_TO_ANGSTROM;
            translationVectors[numTranslationVectors++].Set(x, y,z);
/*
            translationVectors[numTranslationVectors].x =
               atof((char*)vs[column].c_str()) * BOHR_TO_ANGSTROM;
            translationVectors[numTranslationVectors].y =
               atof((char*)vs[column+1].c_str()) * BOHR_TO_ANGSTROM;
            translationVectors[numTranslationVectors].z =
               atof((char*)vs[column+2].c_str()) * BOHR_TO_ANGSTROM;
            numTranslationVectors++;
*/
            ifs->getline(buffer,MYSIZE);
          }
        }
        else if (strstr(buffer, "Symmetries"))
        {
          tokenize(vs, buffer, "()");
          // Should be something like (#160)
          symmetryCode = atoi(vs[1].substr(1).c_str());
        }
        else if (strstr(buffer, " typat"))
        {
//old          tokenize(vs, buffer);
//old          atomTypes.clear();
//old          for (unsigned int i = 1; i < vs.size(); ++i) {
//old            atomTypes.push_back(atoi(vs[i].c_str()));
//old          }
            atomTypes.clear();
	    int n=0;
//ob	    while( n<=natom){
//ob        tokenize(vs, buffer);
//ob        for (unsigned int i = 1; i < vs.size(); ++i) {
//ob        atomTypes.push_back(atoi(vs[i].c_str()));
//ob        }
//ob        n+=vs.size();
//ob        ifs.getline(buffer,BUFF_SIZE);
//ob        }
	    while( n<natom)
            {
                int i0 = (n==0)?1:0;
	        tokenize(vs, buffer);
	        for (unsigned int i = i0; i < vs.size(); ++i)
		  atomTypes.push_back(atoi(vs[i].c_str()));
	        n+=(int)vs.size()-i0;
	        ifs->getline(buffer,MYSIZE);
	    }
        }
        else if (strstr(buffer, "znucl"))
        {
          tokenize(vs, buffer);
          // make sure znucl is first token
          if (vs[0] != "znucl")
            continue;
          // push back the remaining tokens into atomicNumbers
          atomicNumbers.clear();
          atomicNumbers.push_back(0); // abinit starts typat with 1
          for (unsigned int i = 1; i < vs.size(); ++i)
            atomicNumbers.push_back(int(atof(vs[i].c_str())));
        }
     }
     if(natom == 0)
        *err1 = true;
     if(*err1)
        return false;
     for (int i = 0; i < natom; ++i)
     {
      atom = mol->NewAtom();
      //set atomic number
      int idx = atom->GetIdx();
      int type = atomTypes[idx - 1];
      atom->SetAtomicNum(atomicNumbers[type]);
      // we set the coordinates by conformers in another loop
      }
      mol->EndModify();

      int numConformers = (int) atomPositions.size() / natom;
      for (int i = 0; i < numConformers; ++i)
      {
        double *coordinates = new double[natom * 3];
        for (int j = 0; j < natom; ++j) {
          vet3 currentPosition = atomPositions[i*natom + j];
          coordinates[j*3] = currentPosition.x;
          coordinates[j*3 + 1] = currentPosition.y;
          coordinates[j*3 + 2] = currentPosition.z;
        }
        mol->AddConformer(coordinates);
      }
      // Delete first conformer, created by EndModify, bunch of 0s
      mol->DeleteConformer(0);
      // Set geometry to last one
      mol->SetConformer(mol->NumConformers() - 1);
      mol->ConnectTheDots();
      mol->PerceiveBondOrders();
//TEST
   // set default translation vectors 
      if (numTranslationVectors == 0)
      {
          translationVectors[numTranslationVectors++].Set(1, 0, 0);
          translationVectors[numTranslationVectors++].Set(0, 1, 0);
          translationVectors[numTranslationVectors++].Set(0, 0, 1);
      }
//END TEST
   // Attach unit cell translation vectors if found
      if (numTranslationVectors > 0)
      {
        OBUnitCell* uc = new OBUnitCell;
        uc->SetData(acell[0] * translationVectors[0], acell[1] * translationVectors[1], acell[2] * translationVectors[2]);
        uc->SetOrigin(fileformatInput);
        if (symmetryCode)
          uc->SetSpaceGroup(symmetryCode);
        mol->SetData(uc);
      }
      //mol.SetTitle(title);
      return(true);
  }
  bool ReadMOPAC2012(ifstream *ifs, bool *err1)
  {
     *err1 = false;
     char buffer[MYSIZE];
     bool mopac2012 = false;
     for(int k=0; k<10; k++)
     {
         if(ifs->getline(buffer,MYSIZE))
         {
            if (strstr(buffer,"MOPAC") != NULL)
            {
               string buf1 = buffer;
               size_t found = buf1.find("20");
               if(found != string::npos)
               {
                  string buf2 = buf1.substr(found, 4);
                  int num = atoi(buf2.c_str());
                  if (num >= 2012)
                mopac2012 = true;
               }
               //if (strstr(buffer,"2012") != NULL)
               // mopac2012 = true;
               break;
            }
         }
     }
     
     if(mopac2012)
     {
        OBMol mol;
        double x,y,z;
        OBAtom *atom;
        vector<string> vs;
        
        while (ifs->getline(buffer,MYSIZE))
        {
           if(strstr(buffer,"Error") != NULL)
              *err1 = true;
           if(strstr(buffer,"  CARTESIAN COORDINATES") != NULL)
           {
              mol.Clear();
              mol.BeginModify();
              ifs->getline(buffer,MYSIZE);      // blank
              ifs->getline(buffer,MYSIZE);      // column headings ??
              if(strstr(buffer, " ATOM ") && strstr(buffer, " X "))
              {
                 ifs->getline(buffer,MYSIZE);      // blank
                 ifs->getline(buffer,MYSIZE);
              }
              tokenize(vs,buffer);
              while (vs.size() == 5)
              {
//      printf("%s\n", buffer);
                atom = mol.NewAtom();
#if (OB_VERSION >= OB_VERSION_CHECK(3, 0, 0))
                atom->SetAtomicNum(OBElements::GetAtomicNum(vs[1].c_str()));
#else
                atom->SetAtomicNum(etab.GetAtomicNum(vs[1].c_str()));
#endif
                x = atof((char*)vs[2].c_str());
                y = atof((char*)vs[3].c_str());
                z = atof((char*)vs[4].c_str());
                atom->SetVector(x,y,z);

                if (!ifs->getline(buffer,MYSIZE))
                  break;
                tokenize(vs,buffer);
              }
            }
        }
        mol.ConnectTheDots();
        FromMolToAtm(mol);
     }
     ifs->clear();
     ifs->seekg(0, ifs->beg);
     return mopac2012;
  }

  int obReadFile(const char *input_file, std::ostringstream &out)
  {
     obErrorLog.SetOutputStream(&out);
     obErrorLog.StartErrorWrap();
     ifstream ifs;
     bool ier;
     ifs.open(input_file);
     if(!ifs)
       return -1;                  // error opening file

     if(ReadMOPAC2012(&ifs, &ier))
        return 0;
     OBMol mol;
     if(!ReadABINIT(&ifs, &ier, &mol))
     {
        if(ier)
        {
           out << "Reading a molecule from file " << input_file << " failed.";
           return -3;                 // error reading file
        }
        else
        {
           OpenBabel::OBConversion conv(&ifs);
           OBFormat *inFormat = conv.FormatFromExt(input_file);
           if(!conv.SetInFormat(inFormat))
             return -2;                 // unrecognized format
//        if(!conv.ReadFile(&mol, input_file))
           if(!conv.Read(&mol))
           {
             if(out.str().length() == 0)
                out << "Reading a molecule from file " << input_file << " failed.";
             return -3;                 // error reading file
           }
        }
     }

     FromMolToAtm(mol);

     if (mol.HasData(OBGenericDataType::UnitCell))
     {
         OBUnitCell *uc = (OBUnitCell*)mol.GetData(OBGenericDataType::UnitCell);
         Cell[0] = (float) uc->GetA();
         Cell[1] = (float) uc->GetB();
         Cell[2] = (float) uc->GetC();
         Cell[3] = (float) uc->GetAlpha() ;
         Cell[4] = (float) uc->GetBeta();
         Cell[5] = (float) uc->GetGamma();
         passa_cella_al_fortran_(Cell);
         spgnum = uc->GetSpaceGroupNumber();
         if(spgnum > 0)
         {
            const SpaceGroup *sg = uc->GetSpaceGroup();
            const std::string ssg = sg->GetHMName();
            passa_spg_al_fortran_(&spgnum);
            size_t max_l = MRmin(16, ssg.length());
            for(size_t i=0; i< max_l; i++)
                spg[i] = ssg.at(i);
            passa_spgnam_al_fortran_(spg, 16);
         }
     }

     return 0;
  }

  int obReadSMILE(const char *str, std::ostringstream &out)
  {
     obErrorLog.SetOutputStream(&out);
     obErrorLog.StartErrorWrap();

     OBMol mol;
     OBBuilder builder;
     string SmilesString(str);

     stringstream is(SmilesString);
     OBConversion conv(&is);
     if (conv.SetInFormat("smi") && conv.ReadString(&mol, SmilesString))
     {
         builder.Build(mol);

      // optimize geometry
         OBForceField* pFF =  find_force_field("MMFF94");
         if (!pFF)
         {
             if(out.str().length() == 0)
                out << "Force field MMFF94 not found.";
             return -4;
         }

      // Optimize => Add hydrogens => Optimize
      // Test: 'CC(=O)NC1=CC=C(C=C1)O','C1OC2CCCOC2OC1','O=O','C#C'

         unsigned nats = mol.NumAtoms();
         for (int k=0; k<2; k++) {
              //pFF->SetLogFile(&cout);
              //pFF->SetLogLevel(OBFF_LOGLVL_LOW);

              if (pFF->Setup(mol))
              {
                  pFF->ConjugateGradients(250, 1.0e-4);
                  pFF->UpdateCoordinates(mol);
              }
              else 
              {
                  pFF =  find_force_field("UFF");
                  if (!pFF)
                  {
                      if(out.str().length() == 0)
                         out << "Force field UFF not found.";
                      return -4;
                  }
                  if (pFF->Setup(mol))
                  {
                      pFF->ConjugateGradients(250, 1.0e-4);
                      pFF->UpdateCoordinates(mol);
                  }
                  else
                  {
                      out << "ERROR: could not setup force field." << endl;
                      return -5;
                  }
              }
              mol.AddHydrogens();
              if (nats == mol.NumAtoms()) break; // exit if no hydrogens were added
         }

         FromMolToAtm(mol);
         return 0;
     }
     else
     {
          out << "Cannot convert SMILES" << endl;
          return -1;
     }
  }

  OBMol FromAtmToOBMol()
  {
     OBMol mol = OBMol();
     mol.SetTitle("This file is created by expo");
     for(int i=0; i<natoms; i++)
     {
#if (OB_VERSION >= OB_VERSION_CHECK(2, 2, 99))
         OBAtom *at = mol.NewAtom(i+1);
#else
         OBAtom *at = mol.NewAtom();
#endif
         at->SetAtomicNum(getatomicnumber_(&(atoms.at(i))));
         at->SetVector((double)atoms.at(i).xc[0], (double)atoms.at(i).xc[1],
                       (double)atoms.at(i).xc[2]);
         //mol->AddAtom(*at);
     }
     for(int i=0; i<nbonds; i++) 
         mol.AddBond(bonds.at(i).n1, bonds.at(i).n2, bonds.at(i).ord);
     vector<OBBond*>::iterator bond_it;
/* - corrado
     int k=0;
     for (OBBond *b= mol.BeginBond(bond_it);b;b = mol.NextBond(bond_it))
     {
          b->SetLength((double) bonds.at(k).dist);
          k++;
     }
*/
     return mol;
  }

  int obWriteFile(const char *output_file)
  {
     ofstream ofs(output_file);
     if(!ofs)
       return -1;                  // error opening file

     OpenBabel::OBConversion conv(NULL, &ofs);
     OBFormat *outFormat = conv.FormatFromExt(output_file);
     if(!conv.SetOutFormat(outFormat))
       return -2;                 // unrecognized format

     OBUnitCell *cell = new OBUnitCell();
     cell->SetData((double) Cell[0], (double) Cell[1], (double) Cell[2],
                   (double) Cell[3], (double) Cell[4], (double) Cell[5]);
     std::string sg(spg);
     cell->SetSpaceGroup(sg);

     OBMol mol = FromAtmToOBMol();
//corr-introduced to avoid wront bond stereo flag in mol file
     mol.SetDimension(2);

     if(!conv.Write(&mol))
        return -3;
  return 0;
  }

  int OptimizeGeometry(force_field_type ffopt, int *vconstr, std::ostringstream &out)
  {
    obErrorLog.SetOutputStream(&out);
    obErrorLog.StartErrorWrap();

    OBForceField* pFF = find_force_field(labcode[ffopt.id-1]);
    if (!pFF)
    {
       if(out.str().length() == 0)
          out << "Force field " << labcode[ffopt.id-1] << " not found.";
       return -4;
    }
    OBMol mol = FromAtmToOBMol();
//test-corrado: add bond perception because not always bond order is defined
    mol.PerceiveBondOrders();
//test

    if (ffopt.log)
    {
        pFF->SetLogFile(&cout);
        pFF->SetLogLevel(OBFF_LOGLVL_LOW);
    }
    OBFFConstraints constraints;
    constraints.Clear();
    if (ffopt.constr_atoms)
    {
        for (int i=0; i<ffopt.constr_atoms; i++) 
             constraints.AddAtomConstraint(vconstr[i]);   // fix position 
    }
    if (!pFF->Setup(mol, constraints))
    {
        out << "ERROR: could not setup force field." << endl;
        return -5;
    }
    switch(ffopt.algo)
    {
    case CG_ALGO:
         pFF->ConjugateGradients(ffopt.steps, ffopt.conv);
         break;
    case SD_ALGO:
         pFF->SteepestDescent(ffopt.steps, ffopt.conv);
         break;
    }

    pFF->GetCoordinates(mol);
    FromMolToAtm(mol, false);
    return 0;
  }

}   //end namespace
 
extern "C" int obreadfilec_(char *filenam, char errmsg[500])
{
using namespace obutils;
    ostringstream errore;
    char *pwd = g_get_current_dir();
    char *nome = g_path_get_basename(filenam);
    char *dir = g_path_get_dirname(filenam);
    dir_set_current(dir);
    int ier = obReadFile(nome, errore);
    if(ier)
    {
       size_t lung = MRmin(errore.str().length(), (size_t) 500);
       for(size_t i=0; i<lung; i++)
           errmsg[i] = errore.str().at(i);
    }
    //g_print("%s\n", errore.str().c_str());
    dir_set_current(pwd);
    g_free(pwd);
    g_free(nome);
    g_free(dir);
    return ier;
//    obReadFile(filenam);
}

extern "C" int obwritefilec_(char *filenam, int *n_at, int *nb, atom_type *atm, f_bond_type *bb, float cell[], char *spgn)
{
using namespace obutils;
    natoms = *n_at;
    nbonds = *nb;
    atoms.clear();
    bonds.clear();
    for(size_t i=0; i<(size_t) natoms; i++)
        atoms.push_back(atm[i]);
    for(size_t i=0; i<(size_t) nbonds; i++)
        bonds.push_back(bb[i]);

    for(int i=0; i<6; i++)
        Cell[i] = cell[i];
    for(size_t i=0; i<(size_t) 16; i++)
        spg[i] = spgn[i];

    char *pwd = g_get_current_dir();
    char *nome = g_path_get_basename(filenam);
    char *dir = g_path_get_dirname(filenam);
    dir_set_current(dir);
    int ier = obWriteFile(nome);
    dir_set_current(pwd);
    g_free(pwd);
    g_free(nome);
    g_free(dir);
    return ier;
//    obWriteFile(filenam);
}

extern "C" int oboptimizegeometryc_(force_field_type *ffopt, int *n_at, int *nb, atom_type *atm, f_bond_type *bb, int *vconstr, char errmsg[500])
{
using namespace obutils;
    ostringstream errore;
    natoms = *n_at;
    nbonds = *nb;
    atoms.clear();
    bonds.clear();
    for(size_t i=0; i<(size_t) natoms; i++)
        atoms.push_back(atm[i]);
    for(size_t i=0; i<(size_t) nbonds; i++)
        bonds.push_back(bb[i]);
    int ier = OptimizeGeometry(*ffopt, vconstr, errore);
    if(ier)
    {
       size_t lung = MRmin(errore.str().length(), (size_t) 500);
       for(size_t i=0; i<lung; i++)
           errmsg[i] = errore.str().at(i);
    }
    else
    {
       for(size_t i=0; i<(size_t) natoms; i++)
           for(int j=0; j<3; j++)
               atm[i].xc[j] = atoms.at(i).xc[j];
       for(size_t i=0; i<(size_t) nbonds; i++)
       {
            bb[i].n1   = bonds.at(i).n1;
            bb[i].n2   = bonds.at(i).n2;
            bb[i].dist = bonds.at(i).dist;
            bb[i].ord  = bonds.at(i).ord;
       }
       if(ffopt->log)
       {
          size_t lung = MRmin(errore.str().length(), (size_t) 500);
          for(size_t i=0; i<lung; i++)
              errmsg[i] = errore.str().at(i);
       }
    }
    return ier;
}

extern "C" int obreadsmilesc_(char *stringa, char errmsg[500])
{
using namespace obutils;
    ostringstream errore;
    int ier = obReadSMILE(stringa, errore);
    if(ier)
    {
       size_t lung = MRmin(errore.str().length(), (size_t) 500);
       for(size_t i=0; i<lung; i++)
           errmsg[i] = errore.str().at(i);
    }
    //g_print("%s\n", errore.str().c_str());
    return ier;
}

bool Importa(string nomefile, int whichModel, mat3x3 rmI)
{
using namespace obutils;
    bool ier;
    ifstream ifs(nomefile.c_str());
    if(!ifs)
       return false;
    ReadMOPAC2012(&ifs,  &ier);
    if(ier)
       return false; 
    int code = 1;
    atom_type *atms = g_new0(atom_type, natoms);
    for(int i=0; i<natoms; i++)
    {
        memcpy(&atms[i],&atoms.at(i), sizeof(atom_type));
        vet3 xyzc = MRMatrix3MultiplyVector3(rmI,
            MRVectorMake(atoms.at(i).xc));  //orto -> fraz.
        atms[i].xc[0] = (float) xyzc.x;
        atms[i].xc[1] = (float) xyzc.y;
        atms[i].xc[2] = (float) xyzc.z;
        atms[i].nz    = GetNz(atms[i].nz);
    }
    jav_optimize_geometry_mopac_(&code, &whichModel, atms, &natoms);
    return true; 
}
