MODULE progproject

   implicit none
   character(len=*), parameter :: PRJEXT = 'expo'

CONTAINS

   subroutine writeproject(nameproject)
   USE General, only: StructureName
   USE fileutil
   USE variables, only: cryst,dataset
   USE datasetmod, only: save_data_bin
   USE sannel, only: goptim, save_sa
   USE peak_mod
   USE crystal_phase
   USE datamod
   USE indexing
   USE spgext
   USE mpi_prog
   USE exploreset
   USE model_util
   character(len=*), intent(in)  :: nameproject
   integer                       :: unitp
   type(file_handle)             :: fileh
   character(len=:), allocatable :: namep
!
#ifdef MPI
   if (mpi_prog_rank /= mpi_prog_master) return
#endif
   if (len_trim(nameproject) == 0) then
       namep = trim(StructureName)//'.expo'
   else
       namep = nameproject
   endif
   call fileh%fopen(namep,ios='wb')
   if (fileh%good()) then
       unitp = fileh%handle()
!
       call save_general_info(unitp)
!
       call save_crystal_bin(cryst,unitp)
!
       call save_data_bin(dataset,unitp)
!
!      save from SA
!!!!!! FIXME salvare le condizioni di SA
       call save_sa(unitp,goptim)
!
!      save from alltrials
       call save_models_bin(unitp,modelset) 
!
!      Rietveld
!!!TODO:       call save_restraints(unitp,resvr)
!
!      save for indexing
       call save_peaks_bin(unitp,pkind)
       call write_cell_info_bin(unitp)
!
!      save for findspace
       call save_spglist_bin(spglist,unitp)
!
       call upd_recProjects(trim(namep) // char(0))
   else
!FIXME - migliora usando handle_open_error
       write(0,*)'error opening file'//trim(namep)
   endif
   end subroutine writeproject

!-----------------------------------------------------------------------------------------------

   integer function readproject(nameproject)  result(ierp)
   USE fileutil
   USE variables, only: cryst,dataset
   USE sinterface, only: crystal_to_expo
   USE ilmviewmod
   USE drawatoms
   USE General, only: jobname
   USE conteggi, only:npunti
   USE datasetmod, only: read_data_bin, POW_DATA, SCRYS_DATA
   USE view
   USE enable_amb
   USE messagemod
   USE proginterface
   USE sannel, only: goptim, read_sa
   USE errormod
   USE molcom, only:jscreen
   USE peak_mod
   USE crystal_phase
   USE indexing
   USE spgext
   USE comandiexpo, only: icomat
   USE commandsmod
   USE plotstyle
   USE model_util
   USE exploreset
   character(len=*), intent(in) :: nameproject
   type(file_handle)            :: fileh
   integer                      :: unitp
   type(error_type)             :: errp
!
   call fileh%fopen(nameproject,'b')
   if (fileh%good()) then

       unitp = fileh%handle()

       call read_general_info(unitp,errp)
       if (errp%signal) go to 10
           
       call read_crystal_bin(cryst,unitp,errp)
       if (errp%signal) go to 10

       call read_data_bin(dataset,unitp,errp)
       if (errp%signal) go to 10

       call read_sa(unitp,goptim,errp)
       if(errp%signal) go to 10

       call read_models_bin(unitp,modelset,errp)
       if(errp%signal) go to 10
!
!      Find additional info in binary, error is not important
       call read_peaks_bin(unitp,pkind,errp)
       if(errp%signal) go to 20
       call read_cell_info_bin(unitp,errp)
       if(errp%signal) go to 20
       call read_spglist_bin(spglist,unitp,errp)
       if(errp%signal) go to 20

20     if (errp%signal) then
           call errp%as_warn()  ! Set error as warning
           call errp%add(' from project file '//trim(nameproject))
           call errp%print()
       endif

       call init_dati()  ! necessary for compatibility with old bin file

       call crystal_to_expo(cryst(1))
       call dataset_to_expo(dataset(1))
!
!      In case of single crystal, to have alignment with lines in machkls (data.f)
       if (dataset(1)%datatype == SCRYS_DATA) then
           if (cryst(1)%numref() > 0) then
               dataset(1)%tmin = minval(cryst(1)%ref(:)%tthd(1))
               dataset(1)%tmax = maxval(cryst(1)%ref(:)%tthd(1))
           endif
       endif
!
       if (jscreen > 0) then
           call AbilitaPaned(1)
           call init_messages()
           call SetWindowTitle(trim(jobname)//' ['//trim(nameproject)//']')
           call write_message_pattern(nameproject)
       endif
       if (dataset(1)%npoints() > 0) then
           if (jscreen > 0) then
               if (dataset(1)%datatype == POW_DATA) then
                   call set_plot_style(8)
                   if (numpeaks(pkind) > 0) then
                       style(STYLE_PEAKS)%vis = 1
                   endif
                   call abilita_tasti('pattern')
                   call vedinew(8,1,npunti,lsetstyle=.false.)
               endif
           endif
           if (next_command_is(commands,icomat,NTREOR_CMD) .or.  &
               next_command_is(commands,icomat,DICVOL_CMD) .or.  &
               next_command_is(commands,icomat,MCMAILLE_CMD)) then
               call toolbar_set_state('continue'//char(0),1)
               call data_for_index()
           endif
       endif
       if (jscreen > 0 .and. cryst(1)%natoms() > 0) then
           call ilmview_open(nameproject)
           call draw_atoms(cryst(1)%at,cryst(1)%bond)
           call abilita_tasti('structure')
       endif
       call fileh%fclose()
       ierp = 0
   else
       ierp = 1
   endif

   return

10 call errp%add(' from project file '//trim(nameproject))
   call errp%print()
   ierp = 2

   end function readproject

!-----------------------------------------------------------------------------------------------

   subroutine save_general_info(unitp)
   USE General
   integer, intent(in) :: unitp
   write(unitp) fname(:), StructureName, jobname
   write(unitp) knw
!corr        write(0,*)'fout=',fname(OUT_FILE)
   end subroutine save_general_info

!-----------------------------------------------------------------------------------------------

   subroutine read_general_info(unitp,err)
   USE General
   USE errormod
   integer, intent(in)           :: unitp
   type(error_type), intent(out) :: err
   integer                       :: ier
   read(unitp, iostat=ier) fname(:), StructureName, jobname
   read(unitp, iostat=ier) knw
   if (ier /= 0) then
       call err%set('Error on reading general information')
   endif
   end subroutine read_general_info

END MODULE progproject

!-----------------------------------------------------------------------------------------------
   subroutine writeprojectwin(nameproject)
   USE progproject
   USE strutil
   implicit none
   character(len=*), intent(in) :: nameproject
!
   call writeproject(remc0(nameproject))
!
   end subroutine writeprojectwin

!-----------------------------------------------------------------------------------------------
   integer function readprojectwin(nameproject)  result(ierp)
   USE progproject
   USE strutil
   implicit none
   character(len=*), intent(in) :: nameproject
!
   ierp = readproject(remc0(nameproject))
!
   end function readprojectwin

