module rinterface
!
!  Procedures to interface refinement with GUI
!
   use atom_basic, only: atom_type
   use elements, only: element_type

   implicit none

   type crystal_t
      character(len=300)       :: cname
      real, dimension(6)       :: cell
      integer, dimension(6)    :: cellcod
      character(len=16)        :: spgname
      real                     :: scal
      integer                  :: scalcod
      integer                  :: kopcorr = 0
      integer, dimension(3)    :: hklpo
      integer                  :: gcode
      real                     :: gpar = 1.0
      logical                  :: riding_model
      real                     :: adp_factor = 1.2
   end type crystal_t

   type dataset_t
     real                      :: zero=0.0
     real                      :: sdisp=0.0
     real                      :: stran=0.0
     integer                   :: zcode,sdcode,stcode
     integer                   :: bcode
   end type dataset_t

   type(crystal_t), dimension(:), allocatable :: cryph
   type(dataset_t), dimension(:), allocatable :: datas

   private :: new_crystal_t, numcryst

contains

   subroutine make_crystal_t()
   use variables, only: cryst
   use elements
   use atom_type_util
   use crystal_phase
   integer :: nph
!
   call new_crystal_t(cryph,numphase(cryst))
   do nph=1,numphase(cryst)
      cryph(nph)%cname = cryst(nph)%cr_name
      cryph(nph)%cell = cryst(nph)%get_cellpar()
      cryph(nph)%cellcod = cryst(nph)%get_cellcod()
      cryph(nph)%spgname = cryst(nph)%spg%symbol_xhm
      cryph(nph)%scal = cryst(nph)%scal
      cryph(nph)%scalcod = cryst(nph)%scode
      cryph(nph)%riding_model = cryst(nph)%riding_model
      cryph(nph)%adp_factor = cryst(nph)%adp_factor
   enddo
!
   end subroutine make_crystal_t

!----------------------------------------------------------------------------------------------------

   subroutine make_dataset_t()
   USE variables, only: dataset
   integer :: nd
   call new_dataset_t(datas,1)
   do nd=1,1
      datas(nd)%zero = dataset(nd)%zero
      datas(nd)%sdisp = dataset(nd)%sdisp
      datas(nd)%stran = dataset(nd)%stran

      datas(nd)%zcode = dataset(nd)%zcode
      datas(nd)%sdcode = dataset(nd)%sdcode
      datas(nd)%stcode = dataset(nd)%stcode

      !datas(nd)%cond = dataset(nd)%cond
      datas(nd)%bcode = dataset(nd)%bcode(1)
   enddo
   end subroutine make_dataset_t

!----------------------------------------------------------------------------------------------------

   subroutine get_info_crystal(numph,natoms,nelem)
   use variables, only: cryst
   use crystal_phase
   use arrayutil
   integer, intent(out)                            :: numph
   integer, dimension(:), allocatable, intent(out) :: natoms,nelem
   integer                                         :: nph
   numph = numphase(cryst)
   call new_array(natoms,numph)
   call new_array(nelem,numph)
   do nph=1,numph
      natoms(nph) = cryst(nph)%natoms()
      nelem(nph) = cryst(nph)%numelem()
   enddo
   end subroutine get_info_crystal

!----------------------------------------------------------------------------------------------------

   subroutine print_crystal_t(kpr)
   use elements
   use atom_type_util
   integer, intent(in) :: kpr
   integer             :: nph
!
   do nph=1,numcryst(cryph)
      write(kpr,'(a)') 'Name: '//trim(cryph(nph)%cname)
      write(kpr,'(a,6f12.5)') 'Cell: ',cryph(nph)%cell(:)
      write(kpr,'(a,6i12)')   'Code: ',cryph(nph)%cellcod(:)
      write(kpr,'(a)') 'Space group: '//trim(cryph(nph)%spgname)
      write(kpr,'(a,f0.3,a,i0)')   'Scale: ',cryph(nph)%scal,' Code: ',cryph(nph)%scalcod
!!mr      call print_elements(cryph(nph)%elem,kpr)
!!mr      call print_atoms(cryph(nph)%at,kpr=kpr)
   enddo
!
   end subroutine print_crystal_t

!----------------------------------------------------------------------------------------------------

   integer function numcryst(phase)
   type(crystal_t), dimension(:), allocatable, intent(in) :: phase
   if (allocated(phase)) then
       numcryst = size(phase)
   else
       numcryst = 0
   endif
   end function numcryst

!----------------------------------------------------------------------------------------------------

   subroutine new_crystal_t(vetr,n)
   type(crystal_t), allocatable, intent(inout) :: vetr(:)
   integer, intent(in)                         :: n

   if (n < 0) return
   if (numcryst(vetr) /= n) then
       if (allocated(vetr))deallocate(vetr)
       if (n > 0) allocate(vetr(n))
   endif

   end subroutine new_crystal_t

!----------------------------------------------------------------------------------------------------

   integer function numdats(dat)
   type(dataset_t), dimension(:), allocatable, intent(in) :: dat
   if (allocated(dat)) then
       numdats = size(dat)
   else
       numdats = 0
   endif
   end function numdats

!----------------------------------------------------------------------------------------------------

   subroutine new_dataset_t(vetr,n)
   type(dataset_t), allocatable, intent(inout) :: vetr(:)
   integer, intent(in)                         :: n

   if (n < 0) return
   if (numdats(vetr) /= n) then
       if (allocated(vetr))deallocate(vetr)
       if (n > 0) allocate(vetr(n))
   endif

   end subroutine new_dataset_t

end module rinterface

   subroutine finishriet_glob(rfc,bc,czero,zc,sdisp,sdcode,stran,stcode,backc)
!
!  Run refinement and pass global variables from fortran to C
!
   USE RefinecomRef
   USE General, only:is_extraction
   USE background
   USE variables, only: dataset         !,cryst
   USE crystal_phase
         !use constraints
   implicit none
   type (refine_condition_type)            :: rfc   
   integer                                 :: bc, zc, sdcode, stcode
   real                                    :: czero,sdisp,stran
   type(back_condition_type)               :: backc
!
!!!!!prova - pawley
        !if(rfcond%raction == 1) call pawley(1)
!call print_constraints(cryst(1)%constr,cryst(1)%at,0,.true.)
!write(0,*)'REF=',cryst(1)%constr(1)%ref
   call riet_new(1)
        !if(rfcond%raction == 2) call pawley(0)
   is_extraction = is_extraction .or. rfcond%raction > 0
!
!  corrections
   czero = dataset(1)%zero
   zc = dataset(1)%zcode
   sdisp = dataset(1)%sdisp
   sdcode = dataset(1)%sdcode
   stran = dataset(1)%stran
   stcode = dataset(1)%stcode
!
!  background
   bc = dataset(1)%bcode(1)
   dataset(1)%cond = backc
!
   end subroutine finishriet_glob

!-----------------------------------------------------------------------------

   subroutine finishriet_ph(catm,cell,cellcod,scal,scalcod,cpear,cvoi,ctch, &
                    kopcorr,hklpo,gpar,gcode,ph)
!
!  Pass values from fortran to C after running refinement
!
   USE RefinecomRef
   USE variables, only:cryst, dataset
   USE background
   USE atom_type_util
   USE profile_function
   implicit none
   type(atom_type)          :: catm(*)
   real, dimension(6)       :: cell
   integer, dimension(6)    :: cellcod
   real                     :: scal 
   integer                  :: scalcod
   type(profile_function_t) :: cpear, cvoi, ctch
   integer                  :: kopcorr
   integer, dimension(3)    :: hklpo
   real                     :: gpar
   integer                  :: gcode
   integer, intent(in)      :: ph
   integer                  :: i
!
   do i=1, cryst(ph)%natoms()
      catm(i) = cryst(ph)%at(i)
   end do
   scal = cryst(ph)%scal
   scalcod = cryst(ph)%scode
   cell(:) = cryst(ph)%get_cellpar()
   cellcod(:) = cryst(ph)%get_cellcod()
   cpear = dataset(1)%pear(ph)
   cvoi  = dataset(1)%pvoi(ph)
   ctch  = dataset(1)%tch(ph)
   kopcorr = cryst(ph)%kopcorr
   hklpo = cryst(ph)%hklpo
   gpar = cryst(ph)%gpar
   gcode = cryst(ph)%gcode
!
   end subroutine finishriet_ph

!-----------------------------------------------------------------------------

   subroutine ripassa_valori_glob(rfc,bc,czero,zc,sdisp,sdcode,tran,stcode,backc)
   USE RefinecomRef
   USE background
   USE variables, only: dataset
   implicit none
   type (refine_condition_type) :: rfc   
   integer                      :: zc,bc,sdcode,stcode
   real                         :: czero,sdisp,tran
   type(back_condition_type)    :: backc
!
   rfcond = rfc
!
!  corrections
   dataset(1)%zero = czero   
   dataset(1)%zcode = zc    
   dataset(1)%sdisp = sdisp
   dataset(1)%sdcode = sdcode
   dataset(1)%stran = tran
   dataset(1)%stcode = stcode
!
!  background
   dataset(1)%cond = backc
   dataset(1)%bcode = bc
!
   end subroutine ripassa_valori_glob

!-----------------------------------------------------------------------------

   subroutine ripassa_valori_ph(catm,cell,cellcod,scal,scalcod,cpear,cvoi,  &
                 ctch,kopcorr,hklpo,gpar,gcode,typefun,ph)
   USE atom_basic, only: atom_type
   USE variables, only:cryst, dataset
   USE rinterface
   USE profile_function
   USE constraints
   implicit none
   type(atom_type)          :: catm(*)
   real, dimension(6)       :: cell(6)
   integer, dimension(6)    :: cellcod
   real                     :: scal
   integer                  :: scalcod
   integer                  :: typefun
   integer, intent(in)      :: ph
   integer                  :: i
   type(profile_function_t) :: cpear , cvoi, ctch
   integer                  :: kopcorr
   integer, dimension(3)    :: hklpo
   real                     :: gpar
   integer                  :: gcode
!
   do i=1, cryst(ph)%natoms()
      cryst(ph)%at(i) = catm(i)
   end do
   call cryst(ph)%set_cell(cell)
   do i=1,6
      call cryst(ph)%set_cellcod(i,cellcod(i))
   end do
   cryst(ph)%scal = scal
   cryst(ph)%scode = scalcod
   dataset(1)%typefun(ph) = typefun
   select  case (dataset(1)%typefun(ph))
   case (PEARSON7)
         dataset(1)%pear(ph) = cpear
   case (PVOIG)
         dataset(1)%pvoi(ph) = cvoi
   case (TCHZ)
         dataset(1)%tch(ph)  = ctch
   end select
   cryst(ph)%kopcorr= kopcorr
   cryst(ph)%hklpo  = hklpo  
   cryst(ph)%gpar   = gpar   
   cryst(ph)%gcode  = gcode  
!
!  Check constraints for merge dipendent equations
   call check_constraints(cryst(ph)%constr)
!
   end subroutine ripassa_valori_ph

!-----------------------------------------------------------------------------

   subroutine show_model(k)
   USE variables, only: cryst
   USE ilmviewmod
   implicit none
   integer, intent(in) :: k
!
   active_phase = k
   call ilmview_draw(cryst(k)%at,cryst(k)%bond,cryst(k)%elem,cryst(k)%cell,cryst(k)%spg)
!
   end subroutine show_model

!---------------------------------------------------------------------------------------------   

   subroutine get_constraints(ph) 
! 
!  passa al c i constraints
! 
   USE variables, only: cryst
   USE constraints
   implicit none
   integer :: i,ph
   integer :: nconstr
! 
   nconstr = nconstraints(cryst(ph)%constr)
   call passa_num_constraints(nconstr,ph)
   do i=1, nconstr
      call passa_constr(cryst(ph)%constr(i)%code, cryst(ph)%constr(i)%npar, cryst(ph)%constr(i)%ref,   &
                   cryst(ph)%constr(i)%vpar,cryst(ph)%constr(i)%vat,cryst(ph)%constr(i)%coef,          &
                   cryst(ph)%constr(i)%sumc,i,cryst(ph)%constr(i)%typec,cryst(ph)%constr(i)%npari,ph)            
   end do
! 
   end subroutine get_constraints
            
!--------------------------------------------------------------------------------

   subroutine put_num_constraints(knconstr,ph,riding_model,adp_factor) 
! 
!  get from C the number of constraints and riding variables for the phase ph
!  prende dal c il numero di constraints e 
! 
   USE constraints
   USE variables, only: cryst
   implicit none
   integer :: knconstr,ph
   logical :: riding_model
   real    :: adp_factor

! 
!  reallocate constraints
   call resize_constraints(cryst(ph)%constr,knconstr)
!
!  update riding variables
   cryst(ph)%riding_model = riding_model  
   cryst(ph)%adp_factor = adp_factor  
! 
   end subroutine put_num_constraints
            
!--------------------------------------------------------------------------------

   subroutine put_constr(code, npar, ref, vpar, vat, coef, sumc, j, typec, npari,ph)
! 
!  prende dal C il j-esimo constraint
! 
   USE constraints
   USE variables, only: cryst
   implicit none
   integer, intent(in)                  :: j
   integer, intent(in)                  :: ref
   integer, intent(in)                  :: code 
   integer, intent(in)                  :: npar
   integer, dimension(npar), intent(in) :: vpar
   integer, dimension(npar), intent(in) :: vat
   real, dimension(npar), intent(in)    :: coef
   real, intent(in)                     :: sumc
   integer, intent(in)                  :: typec,npari
   integer                              :: ph
! 
   cryst(ph)%constr(j+1) = constraint_type(code,vpar,vat,coef,sumc,npar,npari,ref=0,typec=typec)
   if (ref == 1) then
       call refine_constraint(cryst(ph)%constr(j+1),cryst(ph)%at,1)
   endif
! 
   end subroutine put_constr

!---------------------------------------------------------------------------------------------   

   subroutine get_info(rinfo_var)
! 
!  Prende dal C il j-esimo constraint
! 
   USE RefinecomRef
   USE progtype, only: refine_info_type
! 
   implicit none
   type(refine_info_type), intent(out) :: rinfo_var
! 
   rinfo_var = rinfo
! 
   end subroutine get_info
